@@ -10,14 +10,14 @@ from .core import (AdditionOperator, CompositionOperator, DiagonalOperator,
 from .decorators import (aligned, contiguous, inplace, linear, real, square,
                          unitary)
 from .memory import empty
-from .utils import isalias, openmp_num_threads, product, tointtuple
+from .utils import (complex_dtype_for, isalias, openmp_num_threads, product,
+                    tointtuple)
 from .utils.ufuncs import multiply_conjugate
 
 __all__ = ['ConvolutionOperator', 'FFTOperator']
 
 try:
     import pyfftw
-    FFTW_ALIGNMENT = 16
     FFTW_DEFAULT_NUM_THREADS = openmp_num_threads()
     FFTW_WISDOM_FILES = tuple(os.path.join(LOCAL_PATH, 'fftw{0}.wisdom'.format(
                               t)) for t in ['', 'f', 'l'])
@@ -104,13 +104,13 @@ class _FFTWConvolutionOperator(Operator):
             self.__init__([n, fft.H, DiagonalOperator(kernel_fft), fft])
             return
 
-        dtype_ = np.dtype('complex' + str(int(dtype.name[5:]) * 2))
+        dtype_ = complex_dtype_for(dtype)
         shape_ = self._reshape_to_halfstorage(shapein, axes)
         _load_wisdom()
-        alignment = self.flags.alignment_input
+        aligned = self.flags.aligned_input
         contiguous = True
-        with _pool.get(shapein, dtype, alignment, contiguous) as in_:
-            with _pool.get(shape_, dtype_, alignment, contiguous) as out:
+        with _pool.get(shapein, dtype, aligned, contiguous) as in_:
+            with _pool.get(shape_, dtype_, aligned, contiguous) as out:
                 t0 = time.time()
                 fplan = pyfftw.FFTW(in_, out, axes=axes,
                                     flags=[fftw_flag],
@@ -177,9 +177,9 @@ class _FFTWRealConvolutionOperator(Operator):
     def direct(self, input, output):
         shape = self.kernel.shape
         dtype = self.kernel.dtype
-        alignment = self.flags.alignment_input
+        aligned = self.flags.aligned_input
         contiguous = True
-        with _pool.get(shape, dtype, alignment, contiguous) as buf:
+        with _pool.get(shape, dtype, aligned, contiguous) as buf:
             self._fplan.update_arrays(input, buf)
             self._fplan.execute()
             buf *= self.kernel
@@ -244,9 +244,9 @@ class _FFTWRealConvolutionOperator(Operator):
     def _restore_kernel(self):
         shape = self.kernel.shape
         dtype = self.kernel.dtype
-        alignment = self.flags.alignment_input
+        aligned = self.flags.aligned_input
         contiguous = True
-        with _pool.get(shape, dtype, alignment, contiguous) as x:
+        with _pool.get(shape, dtype, aligned, contiguous) as x:
             self.get_kernel(x)
             y = empty(self.shapein, self.dtype)
             self._bplan.update_arrays(x, y)
