@@ -1,16 +1,22 @@
 from __future__ import division
 
+import multiprocessing
 import numexpr
 import numpy as np
-
+try:
+    import pyfftw
+except:
+    pass
+from itertools import izip
 from scipy.sparse.linalg import eigsh
 
 from .decorators import inplace, linear, real, square, symmetric
 from .core import (Operator, BlockRowOperator, BroadcastingOperator,
                    CompositionOperator, DenseOperator, DiagonalOperator,
                    ReductionOperator, DirectOperatorFactory,
-                   ReverseOperatorFactory, Variable, X)
-from .utils import cast, ifirst, tointtuple
+                   ReverseOperatorFactory, Variable, X, _pool)
+from .memory import empty
+from .utils import cast, complex_dtype_for, ifirst, izip_broadcast, tointtuple
 
 __all__ = [
     'BandOperator',
@@ -18,11 +24,11 @@ __all__ = [
     'DiagonalNumexprNonSeparableOperator',
     'DifferenceOperator',
     'EigendecompositionOperator',
-    'EinsteinSumOperator',
     'IntegrationTrapezeWeightOperator',
     'PackOperator',
     'SumOperator',
     'SymmetricBandOperator',
+    'SymmetricBandToeplitzOperator',
     'TridiagonalOperator',
     'UnpackOperator',
 ]
@@ -31,7 +37,7 @@ __all__ = [
 class DiagonalNumexprOperator(DiagonalOperator):
     """
     DiagonalOperator whose diagonal elements are calculated on the fly using
-    the numexpr package and that can be seperated when added or multiplied
+    the numexpr package and that can be separated when added or multiplied
     to a block operator.
 
     Note
@@ -44,8 +50,7 @@ class DiagonalNumexprOperator(DiagonalOperator):
     Example
     -------
     >>> alpha = np.arange(100.)
-    >>> d = SeparableDiagonalNumexprOperator(alpha, '(x/x0)**data',
-                                             {'x':1.2, 'x0':1.})
+    >>> d = DiagonalNumexprOperator(alpha, '(x/x0)**data', {'x':1.2, 'x0':1.})
 
     """
     def __init__(self, data, expr, global_dict=None, var='data', dtype=float,
@@ -718,6 +723,103 @@ class SymmetricBandOperator(Operator):
         return out
 
 
+@real
+@linear
+@symmetric
+@inplace
+class SymmetricBandToeplitzOperator(Operator):
+    """
+    The SymmetricBandToeplitz operator for symmetric band Toeplitz matrices.
+
+    The vector product is implemented using the FFTW library, so it scales
+    as O(nlogn) operations.
+
+    Example
+    -------
+    >>> N = SymmetricBandToeplitzOperator(5, [3, 2, 1, 1])
+    >>> print N.todense().astype(int)
+    [[3 2 1 1 0]
+     [2 3 2 1 1]
+     [1 2 3 2 1]
+     [1 1 2 3 2]
+     [0 1 1 2 3]]
+
+    """
+    def __init__(self, shapein, firstrow, dtype=None, fftw_flag='FFTW_MEASURE',
+                 nthreads=None, **keywords):
+        shapein = tointtuple(shapein)
+        if dtype is None:
+            dtype = float
+        if nthreads is None:
+            nthreads = multiprocessing.cpu_count()
+        firstrow = np.asarray(firstrow, dtype)
+        if firstrow.shape[-1] == 1:
+            self.__class__ = DiagonalOperator
+            self.__init__(firstrow[..., 0], broadcast='rightward',
+                          shapein=shapein, **keywords)
+            return
+        nsamples = shapein[-1]
+        bandwidth = 2 * firstrow.shape[-1] - 1
+        ncorr = firstrow.shape[-1] - 1
+        fftsize = 2
+        while fftsize < nsamples + ncorr:
+            fftsize *= 2
+        with _pool.get(fftsize, dtype, aligned=True,
+                       contiguous=True) as rbuffer:
+            with _pool.get(fftsize // 2 + 1, complex_dtype_for(dtype),
+                           aligned=True, contiguous=True) as cbuffer:
+                fplan = pyfftw.FFTW(
+                    rbuffer, cbuffer, fftw_flags=[fftw_flag], threads=nthreads)
+                bplan = pyfftw.FFTW(
+                    cbuffer, rbuffer, direction='FFTW_BACKWARD',
+                    fftw_flags=[fftw_flag], threads=nthreads)
+                kernel = self._get_kernel(firstrow, fplan, rbuffer, cbuffer,
+                                          ncorr, fftsize, dtype)
+        Operator.__init__(self, shapein=shapein, dtype=dtype, **keywords)
+        self.nsamples = nsamples
+        self.fftsize = fftsize
+        self.bandwidth = bandwidth
+        self.ncorr = ncorr
+        self.fplan = fplan
+        self.bplan = bplan
+        self.kernel = kernel
+        self.fftw_flag = fftw_flag
+        self.nthreads = nthreads
+
+    def direct(self, x, out):
+        with _pool.get(self.fftsize, self.dtype, aligned=True,
+                       contiguous=True) as rbuffer:
+            with _pool.get(
+                self.fftsize // 2 + 1, complex_dtype_for(self.dtype),
+                aligned=True, contiguous=True) as cbuffer:
+                lpad = (self.bandwidth - 1) // 2
+                x = x.reshape((-1, self.nsamples))
+                out = out.reshape((-1, self.nsamples))
+                self.fplan.update_arrays(rbuffer, cbuffer)
+                self.bplan.update_arrays(cbuffer, rbuffer)
+
+                for x_, out_, kernel in izip_broadcast(x, out, self.kernel):
+                    rbuffer[:lpad] = 0
+                    rbuffer[lpad:lpad+self.nsamples] = x_
+                    rbuffer[lpad+self.nsamples:] = 0
+                    self.fplan.execute()
+                    cbuffer *= kernel
+                    self.bplan.execute()
+                    out_[...] = rbuffer[lpad:lpad+self.nsamples]
+
+    def _get_kernel(self, firstrow, fplan, rbuffer, cbuffer, ncorr, fftsize,
+                    dtype):
+        firstrow = firstrow.reshape((-1, ncorr + 1))
+        kernel = empty((firstrow.shape[0], fftsize // 2 + 1), dtype)
+        for f, k in izip(firstrow, kernel):
+            rbuffer[:ncorr+1] = f
+            rbuffer[ncorr+1:-ncorr] = 0
+            rbuffer[-ncorr:] = f[:0:-1]
+            fplan.execute()
+            k[...] = cbuffer.real / fftsize
+        return kernel
+
+
 @real
 @linear
 class DifferenceOperator(Operator):
