@@ -1,19 +1,24 @@
 import numpy as np
-from .core import Operator, BlockRowOperator, CompositionOperator
+from .core import (
+    Operator, BlockOperator, BlockRowOperator, CompositionOperator)
+from .decorators import real
 from .utils import MPI
 
 # these function may be overridden
 sum = np.sum
 dot = np.dot
 
+
+@real
 class NormOperator(Operator):
     commin = None
     commout = MPI.COMM_SELF
     shapeout = ()
     operation = 'sum'
+
     def __init__(self, **keywords):
         Operator.__init__(self, **keywords)
-        self.set_rule('.{BlockOperator}', lambda s,b:s._rule_block(s,b), 
+        self.set_rule(('.', BlockOperator), lambda s, b: s._rule_block(s, b),
                       CompositionOperator)
 
     @staticmethod
@@ -27,6 +32,7 @@ class NormOperator(Operator):
                                 new_axisin=b.new_axisout, commin=b.commout,
                                 commout=self.commout) * b
 
+
 class Norm2Operator(NormOperator):
     def direct(self, input, output):
         output[...] = dot(input, input)
