@@ -49,6 +49,8 @@ class Comm(object):
         return [sendobj]
     def allreduce(self, sendobj=None, recvobj=None, op=SUM):
         return sendobj
+    def bcast(self, obj=None, root=0):
+        return obj
     def gather(self, sendobj=None, recvobj=None, root=0):
         return [sendobj]
     def Allgatherv(self, i, o, op=None):
@@ -68,6 +70,10 @@ class Comm(object):
     def Dup(self):
         return Comm(self.rank, self.size)
     Clone = Dup
+    def Free(self):
+        return
+    def Split(self, color=0, key=0):
+        return Comm(self.rank, self.size)
 
     @classmethod
     def Create_keyval(cls, copy_fn=None, delete_fn=None):
@@ -96,10 +102,10 @@ class Comm(object):
         if keyval not in self._keyvals:
             raise ValueError('Invalid keyval.')
         self._attr[keyval] = attrval
-
+        
     @staticmethod
     def f2py(fcomm):
-         return COMM_SELF
+        return COMM_SELF
     def py2f(self):
         return 0
 
