@@ -1,8 +1,9 @@
 import functools
 import numpy as np
 from nose.plugins.skip import SkipTest
+from numpy.testing import assert_equal
 
-from .misc import all_eq
+from .misc import all_eq, strenum
 
 __all__ = ['assert_eq',
            'assert_in',
@@ -13,11 +14,47 @@ __all__ = ['assert_eq',
            'assert_is_not_instance',
            'assert_is_none',
            'assert_is_not_none',
+           'assert_is_type',
            'assert_raises',
            'skiptest',
            'skiptest_if',
            'skiptest_unless_module']
 
+def assert_same(actual, desired, rtol=2, broadcasting=False):
+    """
+    Compare arrays of floats. The relative error depends on the data type.
+
+    Parameters
+    ----------
+    rtol : float
+        Relative tolerance to account for numerical error propagation.
+    broadcasting : bool, optional
+        If true, allow broadcasting betwee, actual and desired array.
+
+    """
+    actual = np.asarray(actual)
+    desired = np.asarray(desired)
+    if not broadcasting and actual.shape != desired.shape:
+        raise AssertionError(
+            "The actual array shape '{0}' is different from the desired one '{"
+            "1}'.".format(actual.shape, desired.shape))
+    dtype = sorted(arg.dtype for arg in [actual, desired])[0]
+    if dtype.kind in ('b', 'i'):
+        assert_equal(actual, desired)
+        return
+    eps = np.finfo(dtype).eps * rtol
+    same = abs(actual - desired) <= eps * np.maximum(abs(actual), abs(desired))
+    same |= np.isnan(actual) & np.isnan(desired)
+    if not np.all(same):
+        def trepr(x):
+            r = repr(x).split('\n')
+            if len(r) > 3:
+                r = [r[0], r[1], r[2] + ' ...']
+            return '\n'.join(r)
+        raise AssertionError(
+            "Arrays are not equal.\n\n(mismatch {0:.1%})\n x: {1}\n y: {2}"
+            .format(1 - np.mean(same), trepr(actual), trepr(desired)))
+
 def assert_eq(a, b, msg=None):
     """ Assert that the two arguments are (almost) equal. """
     assert all_eq(a, b), msg
@@ -72,6 +109,18 @@ def assert_is_not_none(a, msg=None):
         return
     assert False, str(a) + ' is None' + _get_msg(msg)
 
+def assert_is_type(a, cls, msg=None):
+    """ Assert argument is of a specified type. """
+    if type(cls) is type:
+        cls = (cls,)
+    else:
+        cls = tuple(cls)
+    if any(type(a) is t for t in cls):
+        return
+    raise AssertionError(
+        "{0} is of type '{1}' instead of {2}{3}".format(
+        a, type(a).__name__, strenum(c.__name__ for c in cls), _get_msg(msg)))
+
 def assert_raises(*args, **kwargs):
     np.testing.assert_raises(*args, **kwargs)
 assert_raises.__doc__ = np.testing.assert_raises.__doc__
